% 
% eHSGA: Harmony Search hybridized with Genetic Algorithm

clear, close all; clc;
warning('on','MATLAB:RandStream:ActivatingLegacyGenerators')
warning('on','MATLAB:RandStream:ReadingInactiveLegacyGeneratorState') 

% Specify objective function and bounds
funType = 'MHB';
f       = @(x)objfun(x, funType);
probDim = 2;
xL      = repmat(-10, 1, probDim);
xU      = -xL;

% Set elitist HSGA algorithm parameters
eHSGAparams.HMS      = 40;
eHSGAparams.MaxImp   = 1000;
% eHSGAparams.HMCR = 0.2;
eHSGAparams.HMCRmax  = 0.8;
eHSGAparams.HMCRmin  = 0.2;
eHSGAparams.PARmin   = 0.4;
eHSGAparams.PARmax   = 0.8;
eHSGAparams.b        = (xU - xL)/1000;
eHSGAparams.SC       = 10;

% Randomly initialize HM
initPop              = xL(1) + (xU(1)-xL(1)) .* ...
    rand(eHSGAparams.HMS,probDim);
eHSGAparams.initPop  = initPop;

% Perform minimization
[xbest, fbest, xnew_history, fbest_history] = ...
    eHSGA(f, xL, xU, eHSGAparams);

% Plot results

% Contour plot with iterations marked
x1 = linspace(xL(1), xU(1));
x2 = linspace(xL(2), xU(2));
[X1, X2] = meshgrid(x1, x2);
F = zeros(length(x1), length(x2));
for i = 1:length(x1)
    for j = 1:length(x2)
        F(i,j) = f([X1(i,j), X2(i,j)]);
    end
end
figure
contour(X1,X2,F,75)
xlabel('{\itx}_1')
ylabel('{\itx}_2')
hold on

% Show iterations
ImpStep = 100;
for i = 1:ImpStep:eHSGAparams.MaxImp
    c = [0.7 0.7 0.7]*(1 - i/eHSGAparams.MaxImp); % Vary color gray to black
    plot(xnew_history(i:(i+ImpStep-1),1),xnew_history(i:(i+ImpStep-1),2),...
        '+','MarkerEdgeColor',c,'markersize',3)
   pause(0.01)
end

% Convergence plot
figure
semilogy(fbest_history)
xlabel('{\itImp}')
ylabel('{\itf}({\bfx_{best}})')
set(gca,'YGrid','on')
