function f = objfun(x, type)

if strcmp(type, 'MHB')
    % MHB Modified Himmelblau function.
    f = (x(1)^2 + x(2) - 11)^2 + (x(1) + x(2)^2 - 7)^2 ...
        + 0.1*( (x(1)-3)^2 + (x(2)-2)^2 );
    
elseif strcmp(type, 'other')
    f = 2*x(1).^2 - 4*x(1)*x(2) + x(2).^4 + 2;
    
elseif strcmp(type, 'Rastrigin')
    % Rastrigin' function: several local minima
    % xbest = (0, ..., 0); fbest = 0
    s = 0;
    for j = 1:length(x)
        s = s + (x(j)^2 - 10*cos(2*pi*x(j)));
    end
    f = 10*length(x) + s;
    
elseif strcmp(type, 'griewank')
    sum = 0;
    prod = 1;
    for i = 1:length(x)
        xi = x(i);
        sum = sum + xi^2/4000;
        prod = prod * cos(xi/sqrt(i));
    end
    f = sum - prod + 1;
    
elseif strcmp(type, 'Easom')
    % Easom function: several local minima
    % xbest = (pi, pi); fbest = -1
    f = -cos(x(1))*cos(x(2))*exp(-(x(1)-pi)^2-(x(2)-pi)^2);
    
elseif strcmp(type, 'ackley')
    
    c = 2*pi;
    b = 0.2;
    a = 20;
    
    d = length(x);
    
    sum1 = 0;
    sum2 = 0;
    for ii = 1:d
        xi = x(ii);
        sum1 = sum1 + xi^2;
        sum2 = sum2 + cos(c * xi);
    end
    
    term1 = -a * exp(-b * sqrt(sum1/d));
    term2 = -exp(sum2/d);
    
    f = term1 + term2 + a + exp(1);
    
elseif strcmp(type, 'bukin6')
    
    x1 = x(1);
    x2 = x(2);
    
    term1 = 100 * sqrt(abs(x2 - 0.01 * x1^2));
    term2 = 0.01 * abs(x1 + 10);
    
    f = term1 + term2;
    
elseif strcmp(type, 'drop-wave')
    x1 = x(1);
    x2 = x(2);
    
    frac1 = 1 + cos(12 * sqrt(x1^2 + x2^2));
    frac2 = 0.5*(x1^2 + x2^2) + 2;
    
    f = -frac1/frac2;
    
end
